//
// Copyright (c) 2004 - InfoMus Lab - DIST - University of Genova
//
// InfoMus Lab (Laboratorio di Informatica Musicale)
// DIST - University of Genova
//
// http://www.infomus.dist.unige.it
// news://infomus.dist.unige.it
// mailto:staff@infomus.dist.unige.it
//
// Developer: Gualtiero Volpe
// mailto:volpe@infomus.dist.unige.it
//
// Last modified: 2004-11-10
//
// 2019-07-22 ffgl@subpixels.com
// - use enum PluginType

#include "PluginInfo.h"
#include <assert.h>
#include <stdlib.h>
#include <memory.h>

namespace ffgl
{
extern PluginInfo* g_CurrPluginInfo;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// PluginInfo constructor and destructor
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

PluginInfo::PluginInfo(
	FPCREATEINSTANCEGL* CreateInstance,
	const char* UniqueID,
	const char* PluginName,
	unsigned int APIMajorVersion,
	unsigned int APIMinorVersion,
	unsigned int PluginMajorVersion,
	unsigned int PluginMinorVersion,
	PluginType PluginType,
	const char* Description,
	const char* About,
	unsigned int FreeFrameExtendedDataSize,
	const void* FreeFrameExtendedDataBlock )
{
	// This FFGL SDK is intended for developing plugins based on the FFGL 2.0 specification. Please
	// Update your plugin code to use FFGL 2.0.
	assert( APIMajorVersion >= 2 );

	createInstance = CreateInstance;

	// Filling PluginInfoStruct
	pluginInfo.APIMajorVersion = APIMajorVersion;
	pluginInfo.APIMinorVersion = APIMinorVersion;

	bool bEndFound = false;
	for( int i = 0; ( i < sizeof( pluginInfo.PluginName ) ) && ( !bEndFound ); ++i )
	{
		if( PluginName[ i ] == 0 )
			bEndFound = true;
		( pluginInfo.PluginName )[ i ] = ( bEndFound ) ? 0 : PluginName[ i ];
	}

	bEndFound = false;
	for( int j = 0; ( j < 4 ) && ( !bEndFound ); ++j )
	{
		if( UniqueID[ j ] == 0 )
			bEndFound = true;
		( pluginInfo.PluginUniqueID )[ j ] = ( bEndFound ) ? 0 : UniqueID[ j ];
	}

	pluginInfo.PluginType = PluginType;

	// Filling PluginExtendedInfoStruct
	pluginExtendedInfo.About              = _strdup( About );
	pluginExtendedInfo.Description        = _strdup( Description );
	pluginExtendedInfo.PluginMajorVersion = PluginMajorVersion;
	pluginExtendedInfo.PluginMinorVersion = PluginMinorVersion;
	if( ( FreeFrameExtendedDataSize > 0 ) && ( FreeFrameExtendedDataBlock != nullptr ) )
	{
		memcpy( pluginExtendedInfo.FreeFrameExtendedDataBlock, FreeFrameExtendedDataBlock, FreeFrameExtendedDataSize );
		pluginExtendedInfo.FreeFrameExtendedDataSize = FreeFrameExtendedDataSize;
	}
	else
	{
		pluginExtendedInfo.FreeFrameExtendedDataBlock = nullptr;
		pluginExtendedInfo.FreeFrameExtendedDataSize  = 0;
	}

	g_CurrPluginInfo = this;
}

PluginInfo::~PluginInfo()
{
	free( pluginExtendedInfo.About );
	free( pluginExtendedInfo.Description );
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// PluginInfo methods
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

const PluginInfoStruct* PluginInfo::GetPluginInfo() const
{
	return &pluginInfo;
}

const PluginExtendedInfoStruct* PluginInfo::GetPluginExtendedInfo() const
{
	return &pluginExtendedInfo;
}

FPCREATEINSTANCEGL* PluginInfo::GetFactoryMethod() const
{
	return createInstance;
}

}//End namespace ffgl
